@@ -32,7 +32,7 @@
  * \since QGIS 2.9
  */
 
-class CORE_EXPORT QgsColorEffect : public QgsPaintEffect
+class CORE_EXPORT QgsColorEffect : public QgsPaintEffect SIP_NODEFAULTCTORS
 {
 
   public:
@@ -42,13 +42,13 @@ class CORE_EXPORT QgsColorEffect : public QgsPaintEffect
      * \param map encoded properties string map
      * \returns new QgsColorEffect
      */
-    static QgsPaintEffect *create( const QgsStringMap &map ) SIP_FACTORY;
+    static QgsPaintEffect *create( const QVariantMap &map ) SIP_FACTORY;
 
     QgsColorEffect();
 
     QString type() const override { return QStringLiteral( "color" ); }
-    QgsStringMap properties() const override;
-    void readProperties( const QgsStringMap &props ) override;
+    QVariantMap properties() const override;
+    void readProperties( const QVariantMap &props ) override;
     QgsColorEffect *clone() const override SIP_FACTORY;
 
     /**
@@ -121,7 +121,7 @@ class CORE_EXPORT QgsColorEffect : public QgsPaintEffect
 
     /**
      * Sets whether the effect should colorize a picture.
-     * \param colorizeOn set to true to enable colorization
+     * \param colorizeOn set to TRUE to enable colorization
      * \see colorizeOn
      * \see setColorizeColor
      * \see setColorizeStrength
@@ -130,7 +130,7 @@ class CORE_EXPORT QgsColorEffect : public QgsPaintEffect
 
     /**
      * Returns whether the effect will colorize a picture.
-     * \returns true if colorization is enableds
+     * \returns TRUE if colorization is enabled
      * \see setColorizeOn
      * \see colorizeColor
      * \see colorizeStrength
@@ -139,7 +139,7 @@ class CORE_EXPORT QgsColorEffect : public QgsPaintEffect
 
     /**
      * Sets the color used for colorizing a picture. This is only used if
-     *  setColorizeOn() is set to true.
+     *  setColorizeOn() is set to TRUE.
      * \param colorizeColor colorization color
      * \see colorizeColor
      * \see setColorizeOn
@@ -149,7 +149,7 @@ class CORE_EXPORT QgsColorEffect : public QgsPaintEffect
 
     /**
      * Returns the color used for colorizing a picture. This is only used if
-     * colorizeOn() is set to true.
+     * colorizeOn() is set to TRUE.
      * \returns colorization color
      * \see setColorizeColor
      * \see colorizeOn
@@ -159,7 +159,7 @@ class CORE_EXPORT QgsColorEffect : public QgsPaintEffect
 
     /**
      * Sets the strength for colorizing a picture. This is only used if
-     *  setColorizeOn() is set to true.
+     *  setColorizeOn() is set to TRUE.
      * \param colorizeStrength colorization strength, between 0 and 100
      * \see colorizeStrength
      * \see setColorizeOn
@@ -169,7 +169,7 @@ class CORE_EXPORT QgsColorEffect : public QgsPaintEffect
 
     /**
      * Returns the strength used for colorizing a picture. This is only used if
-     * setColorizeOn() is set to true.
+     * setColorizeOn() is set to TRUE.
      * \returns colorization strength, between 0 and 100
      * \see setColorizeStrength
      * \see colorizeOn
