@@ -40,7 +40,8 @@ class QgsFieldPrivate;
 /**
  * \class QgsField
   * \ingroup core
-  * Encapsulate a field in an attribute table or data source.
+  * \brief Encapsulate a field in an attribute table or data source.
+  *
   * QgsField stores metadata about an attribute field, including name, type
   * length, and if applicable, precision.
   * \note QgsField objects are implicitly shared.
@@ -61,6 +62,7 @@ class CORE_EXPORT QgsField
     Q_PROPERTY( QgsDefaultValue defaultValueDefinition READ defaultValueDefinition WRITE setDefaultValueDefinition )
     Q_PROPERTY( QgsFieldConstraints constraints READ constraints WRITE setConstraints )
     Q_PROPERTY( ConfigurationFlags configurationFlags READ configurationFlags WRITE setConfigurationFlags )
+    Q_PROPERTY( bool isReadOnly READ isReadOnly WRITE setReadOnly )
 
 
   public:
@@ -433,6 +435,20 @@ class CORE_EXPORT QgsField
      */
     QgsEditorWidgetSetup editorWidgetSetup() const;
 
+    /**
+     * Make field read-only if \a readOnly is set to true. This is the case for
+     * providers which support generated fields for instance.
+     * \since QGIS 3.18
+     */
+    void setReadOnly( bool readOnly );
+
+    /**
+     * Returns TRUE if this field is a read-only field. This is the case for
+     * providers which support generated fields for instance.
+     * \since QGIS 3.18
+     */
+    bool isReadOnly() const;
+
 #ifdef SIP_RUN
     SIP_PYOBJECT __repr__();
     % MethodCode
