@@ -17,7 +17,7 @@
 
 
 #include "qgis_core.h"
-#include "qgis.h"
+#include "qgis_sip.h"
 
 #include <QString>
 #include <QVector>
@@ -78,7 +78,7 @@ class CORE_EXPORT QgsLayerDefinition
 
     /**
      * \ingroup core
-     * Class used to work with layer dependencies stored in a XML project or layer definition file
+     * \brief Class used to work with layer dependencies stored in a XML project or layer definition file
      */
     class CORE_EXPORT DependencySorter
     {
