@@ -38,14 +38,14 @@ class QgsFeedback;
 #include "qgsconfig.h"
 constexpr int sFilePrefixLength = CMAKE_SOURCE_DIR[sizeof( CMAKE_SOURCE_DIR ) - 1] == '/' ? sizeof( CMAKE_SOURCE_DIR ) + 1 : sizeof( CMAKE_SOURCE_DIR );
 
-#define QgsSetRequestInitiatorClass(request, _class) request.setAttribute( static_cast< QNetworkRequest::Attribute >( QgsNetworkRequestParameters::AttributeInitiatorClass ), _class ); request.setAttribute( static_cast< QNetworkRequest::Attribute >( QgsNetworkRequestParameters::AttributeInitiatorRequestId ), QString( __FILE__ ).mid( sFilePrefixLength ) + ':' + QString::number( __LINE__ ) + " (" + __FUNCTION__ + ")" );
-#define QgsSetRequestInitiatorId(request, str) request.setAttribute( static_cast< QNetworkRequest::Attribute >( QgsNetworkRequestParameters::AttributeInitiatorRequestId ), QString( __FILE__ ).mid( sFilePrefixLength ) + ':' + QString::number( __LINE__ ) + " (" + __FUNCTION__ + "): " + str );
+#define QgsSetRequestInitiatorClass(request, _class) request.setAttribute( static_cast< QNetworkRequest::Attribute >( QgsNetworkRequestParameters::AttributeInitiatorClass ), _class ); request.setAttribute( static_cast< QNetworkRequest::Attribute >( QgsNetworkRequestParameters::AttributeInitiatorRequestId ), QString(QString( __FILE__ ).mid( sFilePrefixLength ) + ':' + QString::number( __LINE__ ) + " (" + __FUNCTION__ + ")") );
+#define QgsSetRequestInitiatorId(request, str) request.setAttribute( static_cast< QNetworkRequest::Attribute >( QgsNetworkRequestParameters::AttributeInitiatorRequestId ), QString(QString( __FILE__ ).mid( sFilePrefixLength ) + ':' + QString::number( __LINE__ ) + " (" + __FUNCTION__ + "): " + str) );
 #endif
 
 /**
  * \class QgsNetworkRequestParameters
  * \ingroup core
- * Encapsulates parameters and properties of a network request.
+ * \brief Encapsulates parameters and properties of a network request.
  * \since QGIS 3.6
  */
 class CORE_EXPORT QgsNetworkRequestParameters
@@ -143,7 +143,7 @@ class QgsNetworkAccessManager;
  * \brief SSL error handler, used for responding to SSL errors encountered during network requests.
  * \ingroup core
  *
- * QgsSslErrorHandler responds to SSL errors encountered during network requests. The
+ * \brief QgsSslErrorHandler responds to SSL errors encountered during network requests. The
  * base QgsSslErrorHandler class responds to SSL errors only by logging the errors,
  * and uses the default Qt response, which is to abort the request.
  *
@@ -192,7 +192,7 @@ class CORE_EXPORT QgsSslErrorHandler
  * \brief Network authentication handler, used for responding to network authentication requests during network requests.
  * \ingroup core
  *
- * QgsNetworkAuthenticationHandler responds to authentication requests encountered during network requests. The
+ * \brief QgsNetworkAuthenticationHandler responds to authentication requests encountered during network requests. The
  * base QgsNetworkAuthenticationHandler class responds to requests only by logging the request,
  * but does not provide any username or password to allow the request to proceed.
  *
@@ -236,7 +236,7 @@ class CORE_EXPORT QgsNetworkAuthenticationHandler
  * \brief network access manager for QGIS
  * \ingroup core
  *
- * This class implements the QGIS network access manager.  It's a singleton
+ * \brief This class implements the QGIS network access manager.  It's a singleton
  * that can be used across QGIS.
  *
  * Plugins can insert proxy factories and thereby redirect requests to
@@ -356,21 +356,35 @@ class CORE_EXPORT QgsNetworkAccessManager : public QNetworkAccessManager
      *
      * This list consists of the beginning of URL strings which will not use the fallback proxy.
      *
+     * \see noProxyList()
      * \see fallbackProxy()
      * \see setFallbackProxyAndExcludes()
      */
     QStringList excludeList() const;
 
+    /**
+     * Returns the no proxy list.
+     *
+     * This list consists of the beginning of URL strings which will not use any proxy at all
+     *
+     * \see excludeList()
+     * \see fallbackProxy()
+     * \see setFallbackProxyAndExcludes()
+     */
+    QStringList noProxyList() const;
+
     /**
      * Sets the fallback \a proxy and URLs which shouldn't use it.
      *
      * The fallback proxy is used for URLs which no other proxy factory returned proxies for.
      * The \a excludes list specifies the beginning of URL strings which will not use this fallback proxy.
+     * The \a noProxyURLs list specifies the beginning of URL strings which will not use any proxy at all
      *
      * \see fallbackProxy()
      * \see excludeList()
+     * \see noProxyList()
      */
-    void setFallbackProxyAndExcludes( const QNetworkProxy &proxy, const QStringList &excludes );
+    void setFallbackProxyAndExcludes( const QNetworkProxy &proxy, const QStringList &excludes, const QStringList &noProxyURLs );
 
     /**
      * Returns the name for QNetworkRequest::CacheLoadControl.
@@ -395,6 +409,30 @@ class CORE_EXPORT QgsNetworkAccessManager : public QNetworkAccessManager
      */
     void setupDefaultProxyAndCache( Qt::ConnectionType connectionType = Qt::BlockingQueuedConnection );
 
+#ifndef SIP_RUN
+
+    /**
+     * Returns TRUE if all network caching is disabled.
+     *
+     * \see setCacheDisabled()
+     * \note Not available in Python bindings.
+     * \since QGIS 3.18
+     */
+    bool cacheDisabled() const { return mCacheDisabled; }
+
+    /**
+     * Sets whether all network caching should be disabled.
+     *
+     * If \a disabled is TRUE then all caching will be disabled, causing all requests
+     * to be retrieved from the network regardless of the request's attributes.
+     *
+     * \see cacheDisabled()
+     * \note Not available in Python bindings.
+     * \since QGIS 3.18
+     */
+    void setCacheDisabled( bool disabled ) { mCacheDisabled = disabled; }
+#endif
+
     /**
      * Returns whether the system proxy should be used.
      */
@@ -410,6 +448,7 @@ class CORE_EXPORT QgsNetworkAccessManager : public QNetworkAccessManager
 
     /**
      * Sets the maximum timeout \a time for network requests, in milliseconds.
+     * If set to 0, no timeout is set.
      *
      * \see timeout()
      * \since QGIS 3.6
@@ -422,8 +461,8 @@ class CORE_EXPORT QgsNetworkAccessManager : public QNetworkAccessManager
      *
      * This method is safe to call in either the main thread or a worker thread.
      *
-     * If \a forceRefresh is false then previously cached replies may be used for the request. If
-     * it is set to true then a new query is always performed.
+     * If \a forceRefresh is FALSE then previously cached replies may be used for the request. If
+     * it is set to TRUE then a new query is always performed.
      *
      * If an \a authCfg has been specified, then that authentication configuration required will automatically be applied to
      * \a request. There is no need to manually apply the authentication to the request prior to calling
@@ -444,8 +483,8 @@ class CORE_EXPORT QgsNetworkAccessManager : public QNetworkAccessManager
      *
      * This method is safe to call in either the main thread or a worker thread.
      *
-     * If \a forceRefresh is false then previously cached replies may be used for the request. If
-     * it is set to true then a new query is always performed.
+     * If \a forceRefresh is FALSE then previously cached replies may be used for the request. If
+     * it is set to TRUE then a new query is always performed.
      *
      * If an \a authCfg has been specified, then that authentication configuration required will automatically be applied to
      * \a request. There is no need to manually apply the authentication to the request prior to calling
@@ -481,7 +520,7 @@ class CORE_EXPORT QgsNetworkAccessManager : public QNetworkAccessManager
     void requestAboutToBeCreated( QgsNetworkRequestParameters request );
 
     /**
-     * This signal is emitted whenever a pending network reply is finished.
+     * Emitted whenever a pending network reply is finished.
      *
      * The \a reply parameter will contain a QgsNetworkReplyContent object, containing all the useful
      * information relating to the reply, including headers and reply content.
@@ -637,8 +676,10 @@ class CORE_EXPORT QgsNetworkAccessManager : public QNetworkAccessManager
     QList<QNetworkProxyFactory *> mProxyFactories;
     QNetworkProxy mFallbackProxy;
     QStringList mExcludedURLs;
+    QStringList mNoProxyURLs;
     bool mUseSystemProxy = false;
     bool mInitialized = false;
+    bool mCacheDisabled = false;
     static QgsNetworkAccessManager *sMainNAM;
     // ssl error handler, will be set for main thread ONLY
     std::unique_ptr< QgsSslErrorHandler > mSslErrorHandler;
@@ -657,4 +698,3 @@ class CORE_EXPORT QgsNetworkAccessManager : public QNetworkAccessManager
 };
 
 #endif // QGSNETWORKACCESSMANAGER_H
-
