@@ -33,7 +33,7 @@ class QgsProcessingLayerPostProcessorInterface;
 /**
  * \class QgsProcessingContext
  * \ingroup core
- * Contains information about the context in which a processing algorithm is executed.
+ * \brief Contains information about the context in which a processing algorithm is executed.
  *
  * Contextual information includes settings such as the associated project, and
  * expression context.
@@ -54,15 +54,7 @@ class CORE_EXPORT QgsProcessingContext
     /**
      * Constructor for QgsProcessingContext.
      */
-    QgsProcessingContext()
-    {
-      auto callback = [ = ]( const QgsFeature & feature )
-      {
-        if ( mFeedback )
-          mFeedback->reportError( QObject::tr( "Encountered a transform error when reprojecting feature with id %1." ).arg( feature.id() ) );
-      };
-      mTransformErrorCallback = callback;
-    }
+    QgsProcessingContext();
 
     //! QgsProcessingContext cannot be copied
     QgsProcessingContext( const QgsProcessingContext &other ) = delete;
@@ -82,10 +74,16 @@ class CORE_EXPORT QgsProcessingContext
       mTransformContext = other.mTransformContext;
       mExpressionContext = other.mExpressionContext;
       mInvalidGeometryCallback = other.mInvalidGeometryCallback;
+      mUseDefaultInvalidGeometryCallback = other.mUseDefaultInvalidGeometryCallback;
       mInvalidGeometryCheck = other.mInvalidGeometryCheck;
       mTransformErrorCallback = other.mTransformErrorCallback;
       mDefaultEncoding = other.mDefaultEncoding;
       mFeedback = other.mFeedback;
+      mPreferredVectorFormat = other.mPreferredVectorFormat;
+      mPreferredRasterFormat = other.mPreferredRasterFormat;
+      mEllipsoid = other.mEllipsoid;
+      mDistanceUnit = other.mDistanceUnit;
+      mAreaUnit = other.mAreaUnit;
     }
 
     /**
@@ -109,8 +107,8 @@ class CORE_EXPORT QgsProcessingContext
     /**
      * Sets the \a project in which the algorithm will be executed.
      *
-     * This also automatically sets the transformContext() to match
-     * the project's transform context.
+     * This also automatically sets the transformContext(), ellipsoid(), distanceUnit() and
+     * areaUnit() to match the project's settings.
      *
      * \see project()
      */
@@ -118,7 +116,15 @@ class CORE_EXPORT QgsProcessingContext
     {
       mProject = project;
       if ( mProject )
+      {
         mTransformContext = mProject->transformContext();
+        if ( mEllipsoid.isEmpty() )
+          mEllipsoid = mProject->ellipsoid();
+        if ( mDistanceUnit == QgsUnitTypes::DistanceUnknownUnit )
+          mDistanceUnit = mProject->distanceUnits();
+        if ( mAreaUnit == QgsUnitTypes::AreaUnknownUnit )
+          mAreaUnit = mProject->areaUnits();
+      }
     }
 
     /**
@@ -152,6 +158,80 @@ class CORE_EXPORT QgsProcessingContext
      */
     void setTransformContext( const QgsCoordinateTransformContext &context ) { mTransformContext = context; }
 
+    /**
+     * Returns the ellipsoid to use for distance and area calculations.
+     *
+     * \see setEllipsoid()
+     * \since QGIS 3.16
+     */
+    QString ellipsoid() const;
+
+    /**
+     * Sets a specified \a ellipsoid to use for distance and area calculations.
+     *
+     * If not explicitly set, the ellipsoid will default to the project()'s ellipsoid setting.
+     *
+     * \see ellipsoid()
+     * \since QGIS 3.16
+     */
+    void setEllipsoid( const QString &ellipsoid );
+
+    /**
+     * Returns the distance unit to use for distance calculations.
+     *
+     * \see setDistanceUnit()
+     * \see areaUnit()
+     * \since QGIS 3.16
+     */
+    QgsUnitTypes::DistanceUnit distanceUnit() const;
+
+    /**
+     * Sets the \a unit to use for distance calculations.
+     *
+     * If not explicitly set, the unit will default to the project()'s distance unit setting.
+     *
+     * \see distanceUnit()
+     * \see setAreaUnit()
+     * \since QGIS 3.16
+     */
+    void setDistanceUnit( QgsUnitTypes::DistanceUnit unit );
+
+    /**
+     * Returns the area unit to use for area calculations.
+     *
+     * \see setAreaUnit()
+     * \see distanceUnit()
+     * \since QGIS 3.16
+     */
+    QgsUnitTypes::AreaUnit areaUnit() const;
+
+    /**
+     * Sets the \a unit to use for area calculations.
+     *
+     * If not explicitly set, the unit will default to the project()'s area unit setting.
+     *
+     * \see areaUnit()
+     * \see setDistanceUnit()
+     * \since QGIS 3.16
+     */
+    void setAreaUnit( QgsUnitTypes::AreaUnit areaUnit );
+
+    /**
+     * Returns the current time range to use for temporal operations.
+     *
+     * \see setCurrentTimeRange()
+     * \since QGIS 3.18
+     */
+    QgsDateTimeRange currentTimeRange() const;
+
+    /**
+     * Sets the \a current time range to use for temporal operations.
+     *
+     * \see currentTimeRange()
+     * \since QGIS 3.18
+     */
+    void setCurrentTimeRange( const QgsDateTimeRange &currentTimeRange );
+
     /**
      * Returns a reference to the layer store used for storing temporary layers during
      * algorithm execution.
@@ -159,7 +239,7 @@ class CORE_EXPORT QgsProcessingContext
     QgsMapLayerStore *temporaryLayerStore() { return &tempLayerStore; }
 
     /**
-     * Details for layers to load into projects.
+     * \brief Details for layers to load into projects.
      * \ingroup core
      * \since QGIS 3.0
      */
@@ -170,7 +250,7 @@ class CORE_EXPORT QgsProcessingContext
         /**
          * Constructor for LayerDetails.
          */
-        LayerDetails( const QString &name, QgsProject *project, const QString &outputName = QString(), QgsProcessingUtils::LayerHint layerTypeHint = QgsProcessingUtils::UnknownType )
+        LayerDetails( const QString &name, QgsProject *project, const QString &outputName = QString(), QgsProcessingUtils::LayerHint layerTypeHint = QgsProcessingUtils::LayerHint::UnknownType )
           : name( name )
           , outputName( outputName )
           , layerTypeHint( layerTypeHint )
@@ -180,10 +260,24 @@ class CORE_EXPORT QgsProcessingContext
         //! Default constructor
         LayerDetails() = default;
 
-        //! Friendly name for layer, to use when loading layer into project.
+        /**
+         * Friendly name for layer, possibly for use when loading layer into project.
+         *
+         * \warning Instead of directly using this value, prefer to call setOutputLayerName() to
+         * generate a layer name which respects the user's local Processing settings.
+         */
         QString name;
 
-        //! Associated output name from algorithm which generated the layer.
+        /**
+         * Set to TRUE if LayerDetails::name should always be used as the loaded layer name, regardless
+         * of the user's local Processing settings.
+         * \since QGIS 3.16
+         */
+        bool forceName = false;
+
+        /**
+         * Associated output name from algorithm which generated the layer.
+         */
         QString outputName;
 
         /**
@@ -191,17 +285,17 @@ class CORE_EXPORT QgsProcessingContext
          *
          * \since QGIS 3.4
          */
-        QgsProcessingUtils::LayerHint layerTypeHint = QgsProcessingUtils::UnknownType;
+        QgsProcessingUtils::LayerHint layerTypeHint = QgsProcessingUtils::LayerHint::UnknownType;
 
         /**
-         * Layer post-processor. May be nullptr if no post-processing is required.
+         * Layer post-processor. May be NULLPTR if no post-processing is required.
          * \see setPostProcessor()
          * \since QGIS 3.2
          */
         QgsProcessingLayerPostProcessorInterface *postProcessor() const;
 
         /**
-         * Sets the layer post-processor. May be nullptr if no post-processing is required.
+         * Sets the layer post-processor. May be NULLPTR if no post-processing is required.
          *
          * Ownership of \a processor is transferred.
          *
@@ -210,6 +304,13 @@ class CORE_EXPORT QgsProcessingContext
          */
         void setPostProcessor( QgsProcessingLayerPostProcessorInterface *processor SIP_TRANSFER );
 
+        /**
+         * Sets a \a layer name to match this output, respecting any local user settings which affect this name.
+         *
+         * \since QGIS 3.10.1
+         */
+        void setOutputLayerName( QgsMapLayer *layer ) const;
+
         //! Destination project
         QgsProject *project = nullptr;
 
@@ -233,7 +334,7 @@ class CORE_EXPORT QgsProcessingContext
     }
 
     /**
-     * Returns true if the given \a layer (by ID or datasource) will be loaded into the current project
+     * Returns TRUE if the given \a layer (by ID or datasource) will be loaded into the current project
      * upon completion of the algorithm or model.
      * \see layersToLoadOnCompletion()
      * \see setLayersToLoadOnCompletion()
@@ -305,7 +406,7 @@ class CORE_EXPORT QgsProcessingContext
      * \since QGIS 3.0
      */
 #ifndef SIP_RUN
-    void setInvalidGeometryCallback( const std::function< void( const QgsFeature & ) > &callback ) { mInvalidGeometryCallback = callback; }
+    void setInvalidGeometryCallback( const std::function< void( const QgsFeature & ) > &callback ) { mInvalidGeometryCallback = callback; mUseDefaultInvalidGeometryCallback = false; }
 #else
     void setInvalidGeometryCallback( SIP_PYCALLABLE / AllowNone / );
     % MethodCode
@@ -329,7 +430,14 @@ class CORE_EXPORT QgsProcessingContext
      * \see setInvalidGeometryCallback()
      * \since QGIS 3.0
      */
-    SIP_SKIP std::function< void( const QgsFeature & ) > invalidGeometryCallback() const { return mInvalidGeometryCallback; }
+    SIP_SKIP std::function< void( const QgsFeature & ) > invalidGeometryCallback( QgsFeatureSource *source = nullptr ) const;
+
+    /**
+     * Returns the default callback function to use for a particular invalid geometry \a check
+     * \note not available in Python bindings
+     * \since QGIS 3.14
+     */
+    SIP_SKIP std::function< void( const QgsFeature & ) > defaultInvalidGeometryCallbackForCheck( QgsFeatureRequest::InvalidGeometryCheck check, QgsFeatureSource *source = nullptr ) const;
 
     /**
      * Sets a callback function to use when encountering a transform error when iterating
@@ -407,6 +515,7 @@ class CORE_EXPORT QgsProcessingContext
      */
     void pushToThread( QThread *thread )
     {
+      // cppcheck-suppress assertWithSideEffect
       Q_ASSERT_X( QThread::currentThread() == QgsProcessingContext::thread(), "QgsProcessingContext::pushToThread", "Cannot push context to another thread unless the current thread matches the existing context thread affinity" );
       tempLayerStore.moveToThread( thread );
     }
@@ -442,22 +551,109 @@ class CORE_EXPORT QgsProcessingContext
      */
     QgsMapLayer *takeResultLayer( const QString &id ) SIP_TRANSFERBACK;
 
+    /**
+     * Returns the preferred vector format to use for vector outputs.
+     *
+     * This method returns a file extension to use when creating vector outputs (e.g. "shp"). Generally,
+     * it is preferable to use the extension associated with a particular parameter, which can be retrieved through
+     * QgsProcessingDestinationParameter::defaultFileExtension(). However, in some cases, a specific parameter
+     * may not be available to call this method on (e.g. for an algorithm which has only an output folder parameter
+     * and which creates multiple output layers in that folder). In this case, the format returned by this
+     * function should be used when creating these outputs.
+     *
+     * It is the algorithm's responsibility to check whether the returned format is acceptable for the algorithm,
+     * and to provide an appropriate fallback when the returned format is not usable.
+     *
+     * \see setPreferredVectorFormat()
+     * \see preferredRasterFormat()
+     *
+     * \since QGIS 3.10
+     */
+    QString preferredVectorFormat() const { return mPreferredVectorFormat; }
+
+    /**
+     * Sets the preferred vector \a format to use for vector outputs.
+     *
+     * This method sets a file extension to use when creating vector outputs (e.g. "shp"). Generally,
+     * it is preferable to use the extension associated with a particular parameter, which can be retrieved through
+     * QgsProcessingDestinationParameter::defaultFileExtension(). However, in some cases, a specific parameter
+     * may not be available to call this method on (e.g. for an algorithm which has only an output folder parameter
+     * and which creates multiple output layers in that folder). In this case, the format set by this
+     * function will be used when creating these outputs.
+     *
+     * \see preferredVectorFormat()
+     * \see setPreferredRasterFormat()
+     *
+     * \since QGIS 3.10
+     */
+    void setPreferredVectorFormat( const QString &format ) { mPreferredVectorFormat = format; }
+
+    /**
+     * Returns the preferred raster format to use for vector outputs.
+     *
+     * This method returns a file extension to use when creating raster outputs (e.g. "tif"). Generally,
+     * it is preferable to use the extension associated with a particular parameter, which can be retrieved through
+     * QgsProcessingDestinationParameter::defaultFileExtension(). However, in some cases, a specific parameter
+     * may not be available to call this method on (e.g. for an algorithm which has only an output folder parameter
+     * and which creates multiple output layers in that folder). In this case, the format returned by this
+     * function should be used when creating these outputs.
+     *
+     * It is the algorithm's responsibility to check whether the returned format is acceptable for the algorithm,
+     * and to provide an appropriate fallback when the returned format is not usable.
+     *
+     * \see setPreferredRasterFormat()
+     * \see preferredVectorFormat()
+     *
+     * \since QGIS 3.10
+     */
+    QString preferredRasterFormat() const { return mPreferredRasterFormat; }
+
+    /**
+     * Sets the preferred raster \a format to use for vector outputs.
+     *
+     * This method sets a file extension to use when creating raster outputs (e.g. "tif"). Generally,
+     * it is preferable to use the extension associated with a particular parameter, which can be retrieved through
+     * QgsProcessingDestinationParameter::defaultFileExtension(). However, in some cases, a specific parameter
+     * may not be available to call this method on (e.g. for an algorithm which has only an output folder parameter
+     * and which creates multiple output layers in that folder). In this case, the format set by this
+     * function will be used when creating these outputs.
+     *
+     * \see preferredRasterFormat()
+     * \see setPreferredVectorFormat()
+     *
+     * \since QGIS 3.10
+     */
+    void setPreferredRasterFormat( const QString &format ) { mPreferredRasterFormat = format; }
+
   private:
 
-    QgsProcessingContext::Flags mFlags = nullptr;
+    QgsProcessingContext::Flags mFlags = QgsProcessingContext::Flags();
     QPointer< QgsProject > mProject;
     QgsCoordinateTransformContext mTransformContext;
+
+    QString mEllipsoid;
+    QgsUnitTypes::DistanceUnit mDistanceUnit = QgsUnitTypes::DistanceUnknownUnit;
+    QgsUnitTypes::AreaUnit mAreaUnit = QgsUnitTypes::AreaUnknownUnit;
+
+    QgsDateTimeRange mCurrentTimeRange;
+
     //! Temporary project owned by the context, used for storing temporarily loaded map layers
     QgsMapLayerStore tempLayerStore;
     QgsExpressionContext mExpressionContext;
+
     QgsFeatureRequest::InvalidGeometryCheck mInvalidGeometryCheck = QgsFeatureRequest::GeometryNoCheck;
+    bool mUseDefaultInvalidGeometryCallback = true;
     std::function< void( const QgsFeature & ) > mInvalidGeometryCallback;
+
     std::function< void( const QgsFeature & ) > mTransformErrorCallback;
     QString mDefaultEncoding;
     QMap< QString, LayerDetails > mLayersToLoadOnCompletion;
 
     QPointer< QgsProcessingFeedback > mFeedback;
 
+    QString mPreferredVectorFormat;
+    QString mPreferredRasterFormat;
+
 #ifdef SIP_RUN
     QgsProcessingContext( const QgsProcessingContext &other );
 #endif
@@ -467,7 +663,7 @@ Q_DECLARE_OPERATORS_FOR_FLAGS( QgsProcessingContext::Flags )
 
 
 /**
- * An interface for layer post-processing handlers for execution following a processing algorithm operation.
+ * \brief An interface for layer post-processing handlers for execution following a processing algorithm operation.
  *
  * Note that post-processing of a layer will ONLY occur if that layer is set to be loaded into a QGIS project
  * on algorithm completion. See QgsProcessingContext::layersToLoadOnCompletion().
