@@ -83,7 +83,7 @@ class CORE_EXPORT QgsImageFetcher : public QObject
 
 /**
  * \ingroup core
- * Base class for raster data providers.
+ * \brief Base class for raster data providers.
  */
 class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRasterInterface
 {
@@ -100,7 +100,9 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
       NoProviderCapabilities = 0,       //!< Provider has no capabilities
       ReadLayerMetadata = 1 << 1, //!< Provider can read layer metadata from data store. Since QGIS 3.0. See QgsDataProvider::layerMetadata()
       WriteLayerMetadata = 1 << 2, //!< Provider can write layer metadata to the data store. Since QGIS 3.0. See QgsDataProvider::writeLayerMetadata()
-      ProviderHintBenefitsFromResampling = 1 << 3 //!< Provider benefits from resampling and should apply user default resampling settings (since QGIS 3.10)
+      ProviderHintBenefitsFromResampling = 1 << 3, //!< Provider benefits from resampling and should apply user default resampling settings (since QGIS 3.10)
+      ProviderHintCanPerformProviderResampling = 1 << 4, //!< Provider can perform resampling (to be opposed to post rendering resampling) (since QGIS 3.16)
+      ReloadData = 1 << 5 //!< Is able to force reload data / clear local caches. Since QGIS 3.18, see QgsDataProvider::reloadProviderData()
     };
 
     //! Provider capabilities
@@ -114,11 +116,13 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      * The \a uri argument gives a provider-specific uri indicating the underlying data
      * source and it's parameters.
      *
-     * The \a options argument specifies generic provider options.
+     * The \a options argument specifies generic provider options and since QGIS 3.16 creation flags are specified within the \a flags value.
      */
-    QgsRasterDataProvider( const QString &uri, const QgsDataProvider::ProviderOptions &providerOptions = QgsDataProvider::ProviderOptions() );
+    QgsRasterDataProvider( const QString &uri,
+                           const QgsDataProvider::ProviderOptions &providerOptions = QgsDataProvider::ProviderOptions(),
+                           QgsDataProvider::ReadFlags flags = QgsDataProvider::ReadFlags() );
 
-    QgsRasterInterface *clone() const override = 0;
+    QgsRasterDataProvider *clone() const override = 0;
 
     /**
      * Returns flags containing the supported capabilities of the data provider.
@@ -148,11 +152,7 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
     Qgis::DataType sourceDataType( int bandNo ) const override = 0;
 
     //! Returns data type for the band specified by number
-    virtual int colorInterpretation( int bandNo ) const
-    {
-      Q_UNUSED( bandNo )
-      return QgsRaster::UndefinedColorInterpretation;
-    }
+    virtual int colorInterpretation( int bandNo ) const;
 
     QString colorName( int colorInterpretation ) const
     {
@@ -217,10 +217,7 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
     //! Reload data (data could change)
     virtual bool reload() { return true; }
 
-    virtual QString colorInterpretationName( int bandNo ) const
-    {
-      return colorName( colorInterpretation( bandNo ) );
-    }
+    QString colorInterpretationName( int bandNo ) const override;
 
     /**
      * Read band scale for raster value
@@ -447,8 +444,9 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      */
     virtual bool setEditable( bool enabled ) { Q_UNUSED( enabled ) return false; }
 
-    //! Writes into the provider datasource
     // TODO: add data type (may be different from band type)
+
+    //! Writes into the provider datasource
     virtual bool write( void *data, int band, int width, int height, int xOffset, int yOffset )
     {
       Q_UNUSED( data )
@@ -549,7 +547,7 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
     virtual QList< double > nativeResolutions() const;
 
     /**
-     * Returns true if the extents reported by the data provider are not reliable
+     * Returns TRUE if the extents reported by the data provider are not reliable
      * and it's possible that there is renderable content outside of these extents.
      *
      * \since QGIS 3.10.0
@@ -580,6 +578,91 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      */
     virtual QgsPoint transformCoordinates( const QgsPoint &point, TransformType type );
 
+
+    /**
+     * Enable or disable provider-level resampling.
+     *
+     * \return TRUE if success
+     * \since QGIS 3.16
+     */
+    virtual bool enableProviderResampling( bool enable ) { Q_UNUSED( enable ); return false; }
+
+    /**
+     * Returns whether provider-level resampling is enabled.
+     *
+     * \note Resampling is effective only if zoomedInResamplingMethod() and/or
+     * zoomedOutResamplingMethod() return non-nearest resampling.
+     *
+     * \see zoomedInResamplingMethod()
+     * \see zoomedOutResamplingMethod()
+     * \see maxOversampling()
+     *
+     * \since QGIS 3.16
+     */
+    bool isProviderResamplingEnabled() const { return mProviderResamplingEnabled; }
+
+    /**
+     * Resampling method for provider-level resampling.
+     * \since QGIS 3.16
+     */
+    enum class ResamplingMethod
+    {
+      Nearest, //!< Nearest-neighbour resampling
+      Bilinear, //!< Bilinear (2x2 kernel) resampling
+      Cubic,//!< Cubic Convolution Approximation (4x4 kernel) resampling
+      CubicSpline, //!< Cubic B-Spline Approximation (4x4 kernel)
+      Lanczos, //!< Lanczos windowed sinc interpolation (6x6 kernel)
+      Average, //!< Average resampling
+      Mode, //!< Mode (selects the value which appears most often of all the sampled points)
+      Gauss //!< Gauss blurring
+    };
+
+    /**
+     * Set resampling method to apply for zoomed-in operations.
+     *
+     * \return TRUE if success
+     * \since QGIS 3.16
+     */
+    virtual bool setZoomedInResamplingMethod( ResamplingMethod method ) { Q_UNUSED( method ); return false; }
+
+    /**
+     * Returns resampling method for zoomed-in operations.
+     * \since QGIS 3.16
+     */
+    ResamplingMethod zoomedInResamplingMethod() const { return mZoomedInResamplingMethod; }
+
+    /**
+     * Set resampling method to apply for zoomed-out operations.
+     *
+     * \return TRUE if success
+     * \since QGIS 3.16
+     */
+    virtual bool setZoomedOutResamplingMethod( ResamplingMethod method ) { Q_UNUSED( method ); return false; }
+
+    /**
+     * Returns resampling method for zoomed-out operations.
+     * \since QGIS 3.16
+     */
+    ResamplingMethod zoomedOutResamplingMethod() const { return mZoomedOutResamplingMethod; }
+
+    /**
+     * Sets maximum oversampling factor for zoomed-out operations.
+     *
+     * \return TRUE if success
+     * \since QGIS 3.16
+     */
+    virtual bool setMaxOversampling( double factor ) { Q_UNUSED( factor ); return false; }
+
+    /**
+     * Returns maximum oversampling factor for zoomed-out operations.
+     * \since QGIS 3.16
+     */
+    double maxOversampling() const { return mMaxOversampling; }
+
+    void readXml( const QDomElement &filterElem ) override;
+
+    void writeXml( QDomDocument &doc, QDomElement &parentElem ) const override;
+
   signals:
 
     /**
@@ -645,6 +728,18 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
 
     mutable QgsRectangle mExtent;
 
+    //! Whether provider resampling is enabled.
+    bool mProviderResamplingEnabled = false;
+
+    //! Resampling method for zoomed in pixel extraction
+    ResamplingMethod mZoomedInResamplingMethod = ResamplingMethod::Nearest;
+
+    //! Resampling method for zoomed out pixel extraction
+    ResamplingMethod mZoomedOutResamplingMethod = ResamplingMethod::Nearest;
+
+    //! Maximum boundary for oversampling (to avoid too much data traffic). Default: 2.0
+    double mMaxOversampling = 2.0;
+
   private:
 
     /**
