@@ -31,7 +31,7 @@ class QImage;
 
 /**
  * \ingroup core
- * This is a helper class for unit tests that need to
+ * \brief This is a helper class for unit tests that need to
  * write an image and compare it to an expected result
  * or render time.
  */
@@ -42,33 +42,69 @@ class CORE_EXPORT QgsRenderChecker
     /**
      * Constructor for QgsRenderChecker.
      */
-    QgsRenderChecker() = default;
+    QgsRenderChecker();
 
+    /**
+     * Returns the base path containing the reference images.
+     *
+     * This defaults to an internal QGIS test data path, but can be changed via setControlImagePath().
+     *
+     * \see setControlImagePath()
+     */
     QString controlImagePath() const;
 
+    /**
+     * Sets the base \a path containing the reference images.
+     *
+     * \see controlImagePath()
+     * \since QGIS 3.18
+     */
+    void setControlImagePath( const QString &path );
+
+    /**
+     * Returns the HTML report describing the results of the test run.
+     */
     QString report() { return mReport; }
 
-    float matchPercent()
+    /**
+     * Returns the percent of pixels which matched the control image.
+     */
+    float matchPercent() const
     {
       return static_cast<float>( mMismatchCount ) /
              static_cast<float>( mMatchTarget ) * 100;
     }
-    unsigned int mismatchCount() { return mMismatchCount; }
-    unsigned int matchTarget() { return mMatchTarget; }
-    //only records time for actual render part
+
+    /**
+     * Returns the number of pixels which did not match the control image.
+     */
+    unsigned int mismatchCount() const { return mMismatchCount; }
+
+    /**
+     * Returns the total number of pixels in the control image.
+     */
+    unsigned int matchTarget() const { return mMatchTarget; }
+
+    /**
+     * Returns the total elapsed time for the rendering test.
+     *
+     * \note this only records time for actual render part.
+     */
     int elapsedTime() { return mElapsedTime; }
     void setElapsedTimeTarget( int target ) { mElapsedTimeTarget = target; }
 
     /**
-     * Base directory name for the control image (with control image path
-     * suffixed) the path to the image will be constructed like this:
-     * controlImagePath + '/' + mControlName + '/' + mControlName + '.png'
+     * Sets the base directory \a name for the control image (with control image path
+     * suffixed).
+     *
+     * The path to the image will be constructed like this:
+     * controlImagePath() + '/' + control name + '/' + control name + '.png'
      */
     void setControlName( const QString &name );
 
     /**
-     * Prefix where the control images are kept.
-     * This will be appended to controlImagePath
+     * Sets the path prefix where the control images are kept.
+     * This will be appended to controlImagePath().
      */
     void setControlPathPrefix( const QString &name ) { mControlPathPrefix = name + '/'; }
 
@@ -77,15 +113,17 @@ class CORE_EXPORT QgsRenderChecker
     //! Gets an md5 hash that uniquely identifies an image
     QString imageToHash( const QString &imageFile );
 
+    /**
+     * Sets the file name of the rendered image generated by the test.
+     */
     void setRenderedImage( const QString &imageFileName ) { mRenderedImageFile = imageFileName; }
 
     /**
-     * The path of the rendered image can be retrieved through that method.
-     * Will return the path set with setRenderedImage() or generated in runTest()
+     * Returns the path of the rendered image generated by the test.
      *
-     * \returns The path to the rendered image
+     * This method will return either the path set with setRenderedImage() or generated in runTest().
      */
-    QString renderedImage() { return mRenderedImageFile; }
+    QString renderedImage() const { return mRenderedImageFile; }
 
     //! \since QGIS 2.4
     void setMapSettings( const QgsMapSettings &mapSettings );
@@ -132,6 +170,13 @@ class CORE_EXPORT QgsRenderChecker
      */
     bool compareImages( const QString &testName, unsigned int mismatchCount = 0, const QString &renderedImageFile = QString() );
 
+    /**
+     * Test using two arbitrary images at the specified paths for equality.
+     *
+     * \since QGIS 3.18
+     */
+    bool compareImages( const QString &testName, const QString &referenceImageFile, const QString &renderedImageFile, unsigned int mismatchCount = 0 );
+
     /**
      * Gets a list of all the anomalies. An anomaly is a rendered difference
      * file where there is some red pixel content (indicating a render check
@@ -183,6 +228,8 @@ class CORE_EXPORT QgsRenderChecker
     void emitDashMessage( const QgsDartMeasurement &dashMessage );
     void emitDashMessage( const QString &name, QgsDartMeasurement::Type type, const QString &value );
 
+    QString mBasePath;
+
     QString mControlName;
     unsigned int mMismatchCount = 0;
     unsigned int mColorTolerance = 0;
