@@ -21,10 +21,6 @@ __author__ = 'Alexander Bruy'
 __date__ = 'September 2013'
 __copyright__ = '(C) 2013, Alexander Bruy'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '0c5774c0685d4b6f3e92a086234455708d1cd427'
-
 import os
 
 from qgis.PyQt.QtGui import QIcon
@@ -37,33 +33,38 @@ from qgis.core import (QgsRasterFileWriter,
                        QgsProcessingParameterExtent,
                        QgsProcessingParameterString,
                        QgsProcessingParameterNumber,
+                       QgsProcessingParameterBoolean,
                        QgsProcessingParameterRasterDestination)
 from processing.algs.gdal.GdalAlgorithm import GdalAlgorithm
 from processing.algs.gdal.GdalUtils import GdalUtils
 
-
 pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
 
 
 class ClipRasterByExtent(GdalAlgorithm):
-
     INPUT = 'INPUT'
     EXTENT = 'PROJWIN'
+    OVERCRS = 'OVERCRS'
     NODATA = 'NODATA'
     OPTIONS = 'OPTIONS'
     DATA_TYPE = 'DATA_TYPE'
+    EXTRA = 'EXTRA'
     OUTPUT = 'OUTPUT'
 
-    TYPES = ['Use input layer data type', 'Byte', 'Int16', 'UInt16', 'UInt32', 'Int32', 'Float32', 'Float64', 'CInt16', 'CInt32', 'CFloat32', 'CFloat64']
-
     def __init__(self):
         super().__init__()
 
     def initAlgorithm(self, config=None):
+
+        self.TYPES = [self.tr('Use Input Layer Data Type'), 'Byte', 'Int16', 'UInt16', 'UInt32', 'Int32', 'Float32', 'Float64', 'CInt16', 'CInt32', 'CFloat32', 'CFloat64']
+
         self.addParameter(QgsProcessingParameterRasterLayer(self.INPUT,
                                                             self.tr('Input layer')))
         self.addParameter(QgsProcessingParameterExtent(self.EXTENT,
                                                        self.tr('Clipping extent')))
+        self.addParameter(QgsProcessingParameterBoolean(self.OVERCRS,
+                                                        self.tr('Override the projection for the output file'),
+                                                        defaultValue=False))
         self.addParameter(QgsProcessingParameterNumber(self.NODATA,
                                                        self.tr('Assign a specified nodata value to output bands'),
                                                        type=QgsProcessingParameterNumber.Double,
@@ -88,6 +89,13 @@ class ClipRasterByExtent(GdalAlgorithm):
         dataType_param.setFlags(dataType_param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
         self.addParameter(dataType_param)
 
+        extra_param = QgsProcessingParameterString(self.EXTRA,
+                                                   self.tr('Additional command-line parameters'),
+                                                   defaultValue=None,
+                                                   optional=True)
+        extra_param.setFlags(extra_param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
+        self.addParameter(extra_param)
+
         self.addParameter(QgsProcessingParameterRasterDestination(self.OUTPUT,
                                                                   self.tr('Clipped (extent)')))
 
@@ -115,6 +123,7 @@ class ClipRasterByExtent(GdalAlgorithm):
             raise QgsProcessingException('Invalid input layer {}'.format(parameters[self.INPUT] if self.INPUT in parameters else 'INPUT'))
 
         bbox = self.parameterAsExtent(parameters, self.EXTENT, context, inLayer.crs())
+        override_crs = self.parameterAsBoolean(parameters, self.OVERCRS, context)
         if self.NODATA in parameters and parameters[self.NODATA] is not None:
             nodata = self.parameterAsDouble(parameters, self.NODATA, context)
         else:
@@ -130,6 +139,10 @@ class ClipRasterByExtent(GdalAlgorithm):
         arguments.append(str(bbox.xMaximum()))
         arguments.append(str(bbox.yMinimum()))
 
+        crs = inLayer.crs()
+        if override_crs and crs.isValid():
+            arguments.append('-a_srs {}'.format(GdalUtils.gdal_crs_string(crs)))
+
         if nodata is not None:
             arguments.append('-a_nodata {}'.format(nodata))
 
@@ -143,6 +156,10 @@ class ClipRasterByExtent(GdalAlgorithm):
         if options:
             arguments.extend(GdalUtils.parseCreationOptions(options))
 
+        if self.EXTRA in parameters and parameters[self.EXTRA] not in (None, ''):
+            extra = self.parameterAsString(parameters, self.EXTRA, context)
+            arguments.append(extra)
+
         arguments.append(inLayer.source())
         arguments.append(out)
 
